@@ -32,6 +32,7 @@ import javax.servlet.ServletException;
 
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.io.FilenameUtils;
+import org.apache.commons.lang.StringUtils;
 import org.glom.libglom.BakeryDocument.LoadFailureCodes;
 import org.glom.libglom.Document;
 import org.glom.libglom.Glom;
@@ -119,7 +120,6 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 			// get and check the glom files in the specified directory
 			// TODO: Test this:
 			final String[] extensions = { GLOM_FILE_EXTENSION };
-			@SuppressWarnings("unchecked")
 			final List<File> glomFiles = (List<File>) FileUtils
 					.listFiles(documentDir, extensions, true /* recursive */);
 
@@ -142,7 +142,7 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 
 			// Check for a specified default locale,
 			// for table titles, field titles, etc:
-			final String globalLocaleID = config.getProperty("glom.document.locale");
+			final String globalLocaleID = StringUtils.defaultString(config.getProperty("glom.document.locale"));
 
 			// This initialisation method can throw an UnsatisfiedLinkError if the java-libglom native library isn't
 			// available. But since we're checking for the error condition above, the UnsatisfiedLinkError will never be
@@ -192,7 +192,7 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 							config.getProperty("glom.document.password")); // can throw an SQLException
 				}
 
-				if (globalLocaleID != null && !globalLocaleID.isEmpty()) {
+				if (!StringUtils.isEmpty(globalLocaleID)) {
 					configuredDocument.setDefaultLocaleID(globalLocaleID.trim());
 				}
 
@@ -253,6 +253,9 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 
 		for (final String documenTitle : documentMapping.keySet()) {
 			final ConfiguredDocument configuredDoc = documentMapping.get(documenTitle);
+			if (configuredDoc == null)
+				continue;
+
 			try {
 				DataSources.destroy(configuredDoc.getCpds());
 			} catch (final SQLException e) {
@@ -284,11 +287,15 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 	@Override
 	public DocumentInfo getDocumentInfo(final String documentID, final String localeID) {
 
-		final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+		ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+
+		// Avoid dereferencing a null object:
+		if (configuredDoc == null)
+			return new DocumentInfo();
 
 		// FIXME check for authentication
 
-		return configuredDoc.getDocumentInfo(localeID);
+		return configuredDoc.getDocumentInfo(StringUtils.defaultString(localeID));
 
 	}
 
@@ -300,10 +307,12 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 	@Override
 	public LayoutGroup getListViewLayout(final String documentID, final String tableName, final String localeID) {
 		final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+		if (configuredDoc == null)
+			return new LayoutGroup();
 
 		// FIXME check for authentication
 
-		return configuredDoc.getListViewLayoutGroup(tableName, localeID);
+		return configuredDoc.getListViewLayoutGroup(tableName, StringUtils.defaultString(localeID));
 	}
 
 	/*
@@ -332,6 +341,9 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 			final String quickFind, final int start, final int length, final int sortColumnIndex,
 			final boolean isAscending) {
 		final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+		if (configuredDoc == null)
+			return new ArrayList<DataItem[]>();
+
 		if (!configuredDoc.isAuthenticated()) {
 			return new ArrayList<DataItem[]>();
 		}
@@ -348,8 +360,19 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 		final Documents documents = new Documents();
 		for (final String documentID : documentMapping.keySet()) {
 			final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
-			documents.addDocument(documentID, configuredDoc.getDocument().get_database_title(),
-					configuredDoc.getDefaultLocaleID());
+			if (configuredDoc == null)
+				continue;
+
+			final Document glomDocument = configuredDoc.getDocument();
+			if (glomDocument == null) {
+				final String errorMessage = "getDocuments(): getDocument() failed.";
+				Log.fatal(errorMessage);
+				// TODO: throw new Exception(errorMessage);
+				continue;
+			}
+
+			final String localeID = StringUtils.defaultString(configuredDoc.getDefaultLocaleID());
+			documents.addDocument(documentID, glomDocument.get_database_title(localeID), localeID);
 		}
 		return documents;
 	}
@@ -361,7 +384,11 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 	 */
 	@Override
 	public boolean isAuthenticated(final String documentID) {
-		return documentMapping.get(documentID).isAuthenticated();
+		final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+		if (configuredDoc == null)
+			return false;
+
+		return configuredDoc.isAuthenticated();
 	}
 
 	/*
@@ -373,6 +400,11 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 	@Override
 	public boolean checkAuthentication(final String documentID, final String username, final String password) {
 		final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+		if (configuredDoc == null) {
+			Log.error(documentID, "The document could not be found for this ID: " + documentID);
+			return false;
+		}
+
 		try {
 			return configuredDoc.setUsernameAndPassword(username, password);
 		} catch (final SQLException e) {
@@ -390,6 +422,8 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 	public DataItem[] getDetailsData(final String documentID, final String tableName,
 			final TypedDataItem primaryKeyValue) {
 		final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+		if (configuredDoc == null)
+			return new DataItem[0];
 
 		// FIXME check for authentication
 
@@ -412,7 +446,8 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 		// FIXME check for authentication
 
 		final DetailsLayoutAndData initalDetailsView = new DetailsLayoutAndData();
-		initalDetailsView.setLayout(configuredDoc.getDetailsLayoutGroup(tableName, localeID));
+		initalDetailsView
+				.setLayout(configuredDoc.getDetailsLayoutGroup(tableName, StringUtils.defaultString(localeID)));
 		initalDetailsView.setData(configuredDoc.getDetailsData(tableName, primaryKeyValue));
 
 		return initalDetailsView;
@@ -445,6 +480,8 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 			final String relationshipName, final TypedDataItem foreignKeyValue, final int start, final int length,
 			final int sortColumnIndex, final boolean ascending) {
 		final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+		if (configuredDoc == null)
+			return new ArrayList<DataItem[]>();
 
 		// FIXME check for authentication
 
@@ -456,6 +493,8 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 	public int getRelatedListRowCount(final String documentID, final String tableName, final String relationshipName,
 			final TypedDataItem foreignKeyValue) {
 		final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+		if (configuredDoc == null)
+			return 0;
 
 		// FIXME check for authentication
 
@@ -472,6 +511,8 @@ public class OnlineGlomServiceImpl extends RemoteServiceServlet implements Onlin
 	public NavigationRecord getSuitableRecordToViewDetails(final String documentID, final String tableName,
 			final String relationshipName, final TypedDataItem primaryKeyValue) {
 		final ConfiguredDocument configuredDoc = documentMapping.get(documentID);
+		if (configuredDoc == null)
+			return new NavigationRecord();
 
 		// FIXME check for authentication
 
